#pragma once

#include "tipupgraderutil.h"
#include "tipconst.h"
#include "tipupgraderipc.h"
#include "tipupgradermsgwnd.h"

#include "Nelo2.h"
#include "../../common/Nelo2Wrapper.h"
#include "../svn_Info.h"

namespace tipupgrader
{
	class Upgrader
	{
	public:
		Upgrader(HINSTANCE hInstance, std::vector<CString>* cmdParameter):m_receiveCallback(this),m_TrayIconWnd(hInstance),m_CmdParameter(cmdParameter),m_bProgramQuit(FALSE),
			m_upgradeDescription(_T("")), m_deadLine(_T(""))
		{			
			CString strRevisionNumber;
			strRevisionNumber.Format(_T("%d%d"), 1, CSVNInfo::Get_SVN_Rev());
            CNelo2Wrapper::GetInstance().Init(_T("ntalk_upgrader"), strRevisionNumber, true);
			//m_hNELO = NELO_Create(_T("ntalk_upgrader"), strRevisionNumber, _T("Ntalk Upgrader"));
			//NELO_SetReportServer(m_hNELO, _T("ecs.naver.com"), 10007);
			//if(!NELO_StartCrashHandler(m_hNELO))
			//{
			//	int nErrcode = NELO_GetLastError();			
			//}
			//NELO_SetAutoReport(m_hNELO, true);			
		}
		~Upgrader()
		{	
            CNelo2Wrapper::ReleaseInstance();
			//NELO_StopCrashHandler(m_hNELO);
			//NELO_Destroy(m_hNELO);			
		}

		int Run();
		HANDLE RunProgram(LPCTSTR programFilename, LPCTSTR runParameter);
		BOOL RunUpgraderFile(LPCTSTR filepath, LPCTSTR runParameter);

		void SetProgramQuit(BOOL bQuit) { m_bProgramQuit = bQuit; }
		void SetIPCConnect() { m_bIPCConnect = TRUE; }

	private:
		UpgraderUtil m_util;

		CString m_localVersion;
		CString m_serverVersion;
		CString m_checkServerVersion;
		CString m_setupFilePath;

		CString m_upgradeDescription;
		CString m_deadLine;

		CString	m_localLicenseVersion;
		CString	m_urlLicense;
		CString	m_urlLicenseJump;

		tipbase::IPCCopyDataServer m_server;
		ReceiveCallback m_receiveCallback;
		BOOL m_bProgramQuit;
		BOOL m_bIPCConnect;
		TrayMessageHandler m_TrayIconWnd;
		std::vector<CString>* m_CmdParameter;

		//NELO_HANDLE m_hNELO;

	
		BOOL IsVersionChanged();	
		BOOL RunUpdateNTalk(LPCTSTR target); 
		BOOL RunUpdateUpgrader(LPCTSTR target);
		BOOL UpgradeNTalk(LPCTSTR target);
		BOOL NotifyNTalkVersionInfo(LPCTSTR target, BOOL bSameVersion);

		BOOL IsUpdateCancel( LPCTSTR target, HANDLE nsisMutex );

		BOOL UpgradeAgent();
		BOOL SaveVersion(LPCTSTR target);
		BOOL UpdateAgent(CString path);

		int  PrepareToUpdate();

		UINT GetUpdateButtonOption();

		BOOL KillNtalkbizProcess();

		BOOL LoadServerInfo(LPCTSTR target);
		BOOL LoadReCheckServerInfo(LPCTSTR target);
		BOOL LoadLocalInfo(LPCTSTR target);

		CString DownloadZipFile();
		CString DownloadUpdateFile();
		BOOL UnZipFile(CString path, CString tmpDir);

		void StartTrayAnimation();
		void EndTrayAnimation();

		CString DownloadLicenseFile();
		BOOL LicenseInfoCopyToAppData();
		BOOL LoadLocalLicense();

		void MakeParameter(CString& dstParameter, LPCTSTR srcParameter);
		BOOL IsAutoUpgrade();
		BOOL IsSilentUpgrade();
		bool TrayIcoRefresh();
		void DisplayPreviousWindow();
	};

	struct TRAYDATA
	{
		HWND hwnd;
		UINT uID;
		UINT uCallbackMessage;
		DWORD Reserved[2];
		HICON hIcon;
	};

	//=========================================================
	// Tray  μ ̸    ڵ  Լ
	//=========================================================
	static HWND FindTrayToolbarWindow()
	{
		HWND hWnd_ToolbarWindow32 = NULL;
		HWND hWnd_ShellTrayWnd;

		hWnd_ShellTrayWnd = ::FindWindow(_T("Shell_TrayWnd"), NULL);
		if(hWnd_ShellTrayWnd)
		{
			HWND hWnd_TrayNotifyWnd = ::FindWindowEx(hWnd_ShellTrayWnd,NULL,_T("TrayNotifyWnd"), NULL);

			if(hWnd_TrayNotifyWnd)
			{
				HWND hWnd_SysPager = ::FindWindowEx(hWnd_TrayNotifyWnd,NULL,_T("SysPager"), NULL);	// WinXP

				// WinXP  SysPager              
				if(hWnd_SysPager)
				{
					hWnd_ToolbarWindow32 = ::FindWindowEx(hWnd_SysPager, NULL,_T("ToolbarWindow32"), NULL);
				}

				// Win2000  쿡 SysPager   TrayNotifyWnd -> ToolbarWindow32  Ѿ
				else
				{
					hWnd_ToolbarWindow32 = ::FindWindowEx(hWnd_TrayNotifyWnd, NULL,_T("ToolbarWindow32"), NULL);
				}
			}
		}

		return hWnd_ToolbarWindow32;
	}

};
